from plone.app.vocabularies.catalog import CatalogSource
from plone.base import PloneMessageFactory as _
from plone.schemaeditor.fields import FieldFactory
from plone.schemaeditor.interfaces import IFieldEditFormSchema
from plone.schemaeditor.interfaces import IFieldFactory
from z3c.relationfield import interfaces
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.component import adapter
from zope.component import queryUtility
from zope.interface import implementer
from zope.intid.interfaces import IIntIds


@implementer(IFieldFactory)
class RelationFieldFactory(FieldFactory):
    def available(self):
        return queryUtility(IIntIds) is not None


class IRelationFieldSettings(schema.interfaces.IField):
    portal_type = schema.Set(
        title=_("Types"),
        description=_("Allowed target types"),
        value_type=schema.Choice(
            title=_("Type"),
            vocabulary="plone.app.vocabularies.ReallyUserFriendlyTypes",
        ),
        required=False,
    )


@adapter(interfaces.IRelationChoice)
@implementer(IFieldEditFormSchema)
def getRelationChoiceEditFormSchema(field):
    return IRelationFieldSettings


class RelationChoiceEditFormAdapter:
    def __init__(self, field):
        self.field = field

    @property
    def portal_type(self):
        field = self.field
        types = []
        types.extend(field.source.query.get("portal_type") or [])
        return types

    @portal_type.setter
    def portal_type(self, value):
        field = self.field
        if value:
            field.source.query["portal_type"] = list(value)
        elif "portal_type" in field.source.query:
            del field.source.query["portal_type"]


RelationChoiceFactory = RelationFieldFactory(
    RelationChoice, _("Relation Choice"), source=CatalogSource()
)


@adapter(interfaces.IRelationList)
@implementer(IFieldEditFormSchema)
def getRelationListEditFormSchema(field):
    return IRelationFieldSettings


class RelationListEditFormAdapter:
    def __init__(self, field):
        self.field = field

    @property
    def portal_type(self):
        field = self.field.value_type
        types = []
        types.extend(field.source.query.get("portal_type") or [])
        return set(types)

    @portal_type.setter
    def portal_type(self, value):
        field = self.field.value_type
        if value:
            field.source.query["portal_type"] = list(value)
        elif "portal_type" in field.source.query:
            del field.source.query["portal_type"]


RelationListFactory = RelationFieldFactory(
    RelationList,
    _("Relation List"),
    value_type=RelationChoice(title=_("Relation Choice"), source=CatalogSource()),
)



from plone.app.relationfield.testing import FUNCTIONAL_WIDGETS_TESTING
from plone.testing import layered

import doctest
import os
import unittest


optionflags = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests(
        [
            layered(
                doctest.DocFileSuite(
                    os.path.join(os.path.pardir, "supermodel.txt"),
                    optionflags=optionflags,
                ),
                layer=FUNCTIONAL_WIDGETS_TESTING,
            )
        ]
    )
    return suite


if __name__ == "__main__":
    unittest.main(default="test_suite")



from plone.testing import layered
from plone.testing.zca import ZCMLSandbox
from unittest import TestSuite

import doctest
import plone.app.relationfield.tests


ZCML_SANDBOX = ZCMLSandbox(
    filename="test_marshall.zcml", package=plone.app.relationfield.tests
)


def test_suite():
    suite = TestSuite()
    OPTIONFLAGS = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE
    suite.addTest(
        layered(
            doctest.DocFileSuite(
                "../marshaler.rst",
                optionflags=OPTIONFLAGS,
                package="plone.app.relationfield.tests",
            ),
            layer=ZCML_SANDBOX,
        )
    )
    return suite



from z3c.relationfield.relation import _object
from z3c.relationfield.relation import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds


PATCHES = None


def get_from_object(self):
    if getattr(self, "_from_id", None):
        return _object(self._from_id)
    else:
        intids = getUtility(IIntIds)

        # Heya, is there no from_object? Please have a look at #12802

        self._from_id = intids.register(self.__dict__["from_object"])
        return _object(self._from_id)


def set_from_object(self, obj):
    if not obj:
        return
    intids = getUtility(IIntIds)
    self._from_id = intids.register(obj)


RelationValue.from_object = property(get_from_object, set_from_object)



from plone.behavior.interfaces import IBehaviorAssignable
from z3c.relationfield.event import _setRelation
from z3c.relationfield.interfaces import IRelation
from z3c.relationfield.interfaces import IRelationList
from zope.schema import getFields


def extract_relations(obj):
    assignable = IBehaviorAssignable(obj, None)
    if assignable is None:
        return
    for behavior in assignable.enumerateBehaviors():
        if behavior.marker == behavior.interface:
            continue
        for name, field in getFields(behavior.interface).items():
            if IRelation.providedBy(field):
                try:
                    relation = getattr(behavior.interface(obj), name)
                except AttributeError:
                    continue
                yield behavior.interface, name, relation
            if IRelationList.providedBy(field):
                try:
                    rel_list = getattr(behavior.interface(obj), name)
                except AttributeError:
                    continue
                if rel_list is not None:
                    for relation in rel_list:
                        yield behavior.interface, name, relation


def update_behavior_relations(obj, event):
    """Re-register relations in behaviors"""
    for behavior_interface, name, relation in extract_relations(obj):
        _setRelation(obj, name, relation)



from plone.app.relationfield.monkey import PATCHES

import pkg_resources


PATCHES


try:
    pkg_resources.get_distribution("plone.app.contenttypes")
except pkg_resources.DistributionNotFound:
    HAS_CONTENTTYPES = False
else:
    HAS_CONTENTTYPES = True



from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.base import PloneMessageFactory as _
from plone.supermodel import model
from plone.supermodel.directives import fieldset
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.interface import provider


@provider(IFormFieldProvider)
class IRelatedItems(model.Schema):
    """Behavior interface to make a Dexterity type support related items."""

    relatedItems = RelationList(
        title=_("label_related_items", default="Related Items"),
        default=[],
        value_type=RelationChoice(
            title="Related", vocabulary="plone.app.vocabularies.Catalog"
        ),
        required=False,
    )
    form.widget(
        "relatedItems",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "recentlyUsed": True  # Just turn on. Config in plone.app.widgets.
        },
    )

    fieldset("categorization", label=_("Categorization"), fields=["relatedItems"])



from setuptools import find_packages
from setuptools import setup


version = "3.0.3"

setup(
    name="plone.app.relationfield",
    version=version,
    description="Plone support for z3c.relationfield",
    long_description=(open("README.rst").read() + "\n" + open("CHANGES.rst").read()),
    # Get more strings from
    # https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="dexterity relations plone zc.relation",
    author="Alec Mitchell",
    author_email="apm13@columbia.edu",
    url="https://pypi.org/project/plone.app.relationfield",
    license="GPL",
    packages=find_packages(),
    namespace_packages=["plone", "plone.app"],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        "plone.base",
        # Keep Products.CMFCore because z3c.dependencychecker does not yet
        # check transitive dependencies in our config.
        "Products.CMFCore",
        "five.intid",
        "plone.app.intid",
        "plone.app.vocabularies",
        "plone.app.z3cform",
        "plone.autoform",
        "plone.behavior",
        "plone.dexterity",
        "plone.rfc822",
        "plone.schemaeditor",
        "plone.supermodel",
        "plone.uuid",
        "z3c.form",
        "z3c.objpath",
        "z3c.formwidget.query",
        "z3c.relationfield",
        "zc.relation",
    ],
    extras_require={
        "test": [
            "plone.app.contenttypes[test]",
            "plone.app.testing",
            "plone.app.dexterity",
            "plone.formwidget.contenttree",
            "plone.testing",
        ]
    },
)



from plone.app.vocabularies.catalog import CatalogSource
from plone.supermodel.exportimport import BaseHandler
from plone.supermodel.utils import valueToElement
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class RelationChoiceBaseHandler(BaseHandler):
    filteredAttributes = BaseHandler.filteredAttributes.copy()
    filteredAttributes.update(
        {
            "portal_type": "w",
            "source": "rw",
            "vocabulary": "rw",
            "vocabularyName": "rw",
        }
    )

    def __init__(self, klass):
        super().__init__(klass)

        self.fieldAttributes["portal_type"] = schema.List(
            __name__="portal_type",
            title="Allowed target types",
            value_type=schema.Text(title="Type"),
        )

    def _constructField(self, attributes):
        portal_type = (
            attributes.get("portal_type") or attributes.get("portal_types") or []
        )
        if "portal_type" in attributes:
            del attributes["portal_type"]

        if not portal_type:
            attributes["source"] = CatalogSource()
        else:
            attributes["source"] = CatalogSource(portal_type=portal_type)

        return super()._constructField(attributes)

    def write(self, field, name, type, elementName="field"):
        element = super().write(field, name, type, elementName)
        portal_type = []

        portal_type.extend(field.source.query.get("portal_type") or [])

        if portal_type:
            attributeField = self.fieldAttributes["portal_type"]
            child = valueToElement(
                attributeField, portal_type, name="portal_type", force=True
            )
            element.append(child)

        return element


RelationChoiceHandler = RelationChoiceBaseHandler(RelationChoice)
RelationListHandler = BaseHandler(RelationList)



from plone.z3cform import layout
from z3c.form import field
from z3c.form import form
from z3c.form.interfaces import IFormLayer
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList


class ITestForm(IFormLayer):
    multiple = RelationList(
        title="Multiple (Relations field)",
        required=False,
        value_type=RelationChoice(
            title="Multiple", vocabulary="plone.app.vocabularies.Catalog"
        ),
    )
    single = RelationChoice(
        title="Single",
        required=False,
        vocabulary="plone.app.vocabularies.Catalog",
    )


class TestForm(form.EditForm):
    fields = field.Fields(ITestForm)


TestView = layout.wrap_form(TestForm)
